#Analysis of block-level jobs data output by dataprep_blocks.py

#See what proportion of jobs fall into high and low density blocks, as 
#opposed to employment areas
#See what proporiton have 2x jobs than people, 2x people than jobs, mixed, none of either

library(dplyr)
library(reshape2)
library(ggplot2)
library(stringr)
library(scales)

homedir = '<set to directory>'
setwd(homedir)

dir.create('output/block_analysis')

# Set employment block threshold
empcut = 2.0

msadefs = c('msa_2013','msa_2018','cz_1990')
for(mdef in msadefs){
  
  #Clean mdef for output in graphs
  mdef_clean = str_split(mdef,'_',simplify = T)[,1] %>% str_to_upper()
  
  #Get MSA stats
  msas = read.csv(paste('output/msapop/pop_',mdef,'.csv',sep = ''))
  msas$shortname = str_split(msas$msaname,'[-,/]',simplify = T)[,1]
  msas$st = str_split(msas$msaname,'[,]',simplify = T)[,2]
  msas$st = str_replace(msas$st, ' ','')
  msas$st = str_split(msas$st,'-',simplify = T)[,1]
  msas$shortname_st = paste(msas$shortname,msas$st, sep = ', ')
  
  #Get 100 largest MSAs
  m100 = msas %>% arrange(-pop)
  m100 = m100[1:100,'msa']
  msas$top100 = msas$msa %in% c(m100)
  msas$all = T
  
  dir.create(paste('output/block_analysis/',mdef,sep = ''))
  
  #For each city, bring in block data 
  
  for(cb in msas$msa){
    print(cb)
    
    blks = read.csv(paste('output/blocks/blocks_',mdef,'/blks_',mdef,'_',cb,'.csv',sep = ''))
    
    #Convert area into numeric
    blks$ALAND10 = as.numeric(blks$ALAND10)
    
    #Confirm zero employment in places with zero land, no density > 500000
    stopifnot(sum(blks[blks$ALAND10 == 0,'C000'])==0)
    stopifnot(blks$jdens <=500000)
    
    #Categories
    blks$res = blks$pop10 > empcut * blks$C000 & blks$pop10 > 0
    blks$emp = empcut * blks$pop10 < blks$C000 & blks$C000 > 0
    blks$none = blks$pop10 == 0 & blks$C000 == 0 
    blks$mix = blks$pop10 <= empcut * blks$C000 & blks$C000 <= empcut * blks$pop10 & blks$pop10 > 0 & blks$C000 > 0
    stopifnot(blks$res + blks$emp + blks$none + blks$mix == 1)
    
    #Compute total size 
    totals = blks %>% summarise(jobs = sum(C000),pop = sum(pop10),area = sum(ALAND10),jobs_mdens = wtd.quantile(jdens,C000,probs = .5),pop_mdens = wtd.quantile(pdens,pop10,probs = .5))
    msas[msas$msa == cb, 'jobs_tot'] = totals$jobs
    msas[msas$msa == cb, 'pop_tot'] = totals$pop
    msas[msas$msa == cb, 'area_tot'] = totals$area
    msas[msas$msa == cb, 'med_jdens'] = totals$jobs_mdens
    msas[msas$msa == cb, 'med_pdens'] = totals$pop_mdens
  
    #Compute land with no jobs or people
    noneareas = blks %>% filter(none == T) %>% summarise(jobs = sum(C000),pop = sum(pop10),area = sum(ALAND10)) #/ totals
    stopifnot(noneareas$jobs + noneareas$pop == 0)
    msas[msas$msa == cb, 'area_unbuilt'] =  noneareas$area
    msas[msas$msa == cb, 'area_built'] =  totals$area - noneareas$area
    
    #Convert area to built area to compute percentages
    totals$area = totals$area - noneareas$area
    
    #Now drop blks with none
    blks = filter(blks, none == F)
    
    #Compute percentage of jobs in employment areas
    empareas = blks %>% filter(emp == T) %>% summarise(jobs = sum(C000),pop = sum(pop10),area = sum(ALAND10)) / totals%>% select(jobs,pop,area)
    msas[msas$msa == cb, 'jobs_emp'] = empareas$jobs 
    msas[msas$msa == cb, 'pop_emp'] = empareas$pop 
    msas[msas$msa == cb, 'area_emp'] = empareas$area 
    
    #Compute fraction in res areas
    resareas = blks %>% filter(res == T) %>% summarise(jobs = sum(C000),pop = sum(pop10),area = sum(ALAND10)) / totals %>% select(jobs,pop,area)
    msas[msas$msa == cb, 'jobs_res'] = resareas$jobs 
    msas[msas$msa == cb, 'pop_res'] = resareas$pop 
    msas[msas$msa == cb, 'area_res'] = resareas$area 
    
    #Compute fraction mixed
    mixareas = blks %>% filter(mix == T) %>% summarise(jobs = sum(C000),pop = sum(pop10),area = sum(ALAND10)) / totals %>% select(jobs,pop,area)
    msas[msas$msa == cb, 'jobs_mix'] = mixareas$jobs 
    msas[msas$msa == cb, 'pop_mix'] = mixareas$pop 
    msas[msas$msa == cb, 'area_mix'] = mixareas$area 
    
    #Fraction of blocks and area with no jobs and no residents
    msas[msas$msa == cb, 'nojobs_blks'] = nrow(filter(blks, C000 == 0)) / nrow(blks)
    msas[msas$msa == cb, 'nojobs_area'] = filter(blks, C000 == 0) %>% summarise(area = sum(ALAND10)) / totals$area

    msas[msas$msa == cb, 'nopop_blks'] = nrow(filter(blks, pop10 == 0)) / nrow(blks)
    msas[msas$msa == cb, 'nopop_area'] = filter(blks, pop10 == 0) %>% summarise(area = sum(ALAND10)) / totals$area

  }
  
  msas[, 'area_unbuilt_pct'] = msas[, 'area_unbuilt'] / msas[, 'area_tot']
  # msas$shortname = factor(msas$shortname, levels = msas$shortname[order(msas$shortname,decreasing = T)])
  write.csv(msas,paste('output/block_analysis/',mdef,'/blockstats_',mdef,'.csv',sep = ''),row.names = F)

  #Now make plots
  
  #Do versions with all MSAs or just the top 100
  for(samp in c('all','top100')){
    mdat = msas[msas[,samp] == T,]
    mdat$outname = factor(mdat$shortname_st,levels = rev(mdat$shortname_st))
    
    for(type in c('emp','res','mix')){
    for(v in c('jobs','pop')){
      mdat$varint = mdat[,paste(v,type,sep = '_')]
      mdat$varea = mdat[,paste('area_',type,sep = '')]
      
      pdf(paste('output/block_analysis/',mdef,'/blks_',mdef,'_pct_',type,'_',v,'_',samp,'.pdf',sep = ''),height = 5)

      gplt = ggplot(mdat, aes(x = varea, y = varint, label = shortname)) +
        geom_text(size = 1.7) +
        theme_bw() +
        scale_x_continuous(labels = percent, limits = c(0,1), name = 'Percentage of built land area') +
        scale_y_continuous(labels = percent, limits = c(0,1), name = 'Percentage of jobs') +
        ggtitle(paste('Percentage of MSA',v,'in',type,'areas'))

      print(gplt)
      dev.off()

    }
  }
  
  
  #Plot fraction of land with nothing
  if(samp == 'all'){
    pdf(paste('output/block_analysis/',mdef,'/builtland_',mdef,'_',samp,'.pdf',sep = ''),height = 25)
    samptitle = 'all'
  }
    
  if(samp == 'top100'){
    pdf(paste('output/block_analysis/',mdef,'/builtland_',mdef,'_',samp,'.pdf',sep = ''),height = 13,width = 8)
    samptitle = 'largest 100'
  }
    
    gplt = ggplot(mdat, aes(x = area_unbuilt_pct, y = outname)) +
    geom_point() +
    theme_bw() +
    scale_x_continuous(labels = percent, limits = c(0,1), name = paste('Percentage of',mdef_clean,'land area')) +
    scale_y_discrete(name = '') +
    ggtitle(paste('Percentage of',mdef_clean,'land area in blocks with no residents or jobs'))
  
  print(gplt)
  dev.off()
  
  
  #Graph with percent of land and jobs in different types of built area
  pdf(paste('output/block_analysis/',mdef,'/jobpcts_',mdef,'_',samp,'.pdf',sep = ''),height = 5)
  gplt = ggplot(mdat) +
    geom_text(size = 1.7, aes(x = area_emp, y = jobs_emp, label = shortname))+ #,alpha = 1) +
    geom_text(size = 1.7, aes(x = area_res, y = jobs_res, label = shortname))+ #, alpha = .6) +
    geom_text(size = 1.7, aes(x = area_mix, y = jobs_mix, label = shortname))+ #, alpha = .3) +
    theme_bw() +
    scale_x_continuous(labels = percent, limits = c(0,1), name = 'Percentage of built land area') +
    scale_y_continuous(labels = percent, limits = c(0,1), name = 'Percentage of jobs') +
    ggtitle('Percentage of land and jobs in employment, residential, and mixed blocks')

  print(gplt)
  dev.off()
  
  #Graph of median job vs median population density
  pdf(paste('output/block_analysis/',mdef,'/meddens_',mdef,'_',samp,'.pdf',sep = ''),height = 5)
  if(samp == 'top100') {
    limx = 8000
    limy = 12500
  }
  if(samp == 'all'){
    limx = 8000
    limy = 45000
  }
  gplt = ggplot(mdat, aes(x = med_pdens, y = med_jdens, label = shortname)) +
    geom_text(size = 1.7) +
    theme_bw() +
    scale_x_continuous(labels = comma, name = 'Median residential density (people/km2)',limits = c(0,limx)) +
    scale_y_continuous(labels = comma, name = 'Median employment density (jobs/km2)',limits = c(0,limy)) #+
    # ggtitle(paste('Median employment and residential density, ',samptitle,' ',mdef_clean,'s',sep = ''))
  
  print(gplt)
  dev.off()
  }
  
  }
  