'''
Identify business districts
'''

import pandas as pd
import geopandas as gp
import numpy as np
import os
import pysal as ps
import sys

homedir = '<set to directory>'
os.chdir(homedir)

#New Modules
sys.path.append('/Users/wanamingo/projects/cbds/code/modules')


# For alternate specifications, adjust these parameters. 
bufthresh = 15 #Maximum distance in meters between 'adjacent' blocks
empcut = 2.0 #Ratio of jobs to residents to designate employment blocks
    
#UTM zone lookups for projection
utms = {10:'26910',
        11:'26911',
        12:'26912',
        13:'26913',
        14:'26914',
        15:'26915',
        16:'26916',
        17:'26917',
        18:'26918',
        19:'26919',
        1:'26901',
        2:'26902',
        3:'26903',
        4:'26904',
        5:'26905',
        6:'26906',
        7:'26907',
        8:'26908',
        9:'26909'
        }
        
def loadmsas(bufthresh,empcut,mlist,msalist,mdef):
    place = 'b%s_r%s' %(bufthresh,empcut)
    
    #See if weight directory exists and make it if necessary
    try:
        os.stat("output/weights/b%s" %bufthresh)
    except:
        os.mkdir("output/weights/b%s" %bufthresh)

    #See if output directory exists and make if necessary
    try:
        os.stat("output/bizdist_%s" %place)
    except:
        os.mkdir("output/bizdist_%s" %place)
        
    try:
        os.stat("output/bizdist_%s/%s" %(place,mdef))
    except:
        os.mkdir("output/bizdist_%s/%s" %(place,mdef))
        os.mkdir("output/bizdist_%s/%s/bd_shapes" %(place,mdef))
        os.mkdir("output/bizdist_%s/%s/bd_stats" %(place,mdef))
        os.mkdir("output/bizdist_%s/%s/bd_stats/randomize" %(place,mdef))
        os.mkdir("output/bizdist_%s/%s/bd_maps" %(place,mdef))
        os.mkdir("output/bizdist_%s/%s/bd_blks" %(place,mdef))
    
    for msaint in mlist:
        msaname = msalist.loc[msalist['msa'] == msaint, 'msaname'].values[0]

        #Aleutian islands have odd UTM zone, so skip
        if msaint == '34114':
            continue
        getcbds(bufthresh,empcut,place,msaint,msaname,mdef)

def getcbds(bufthresh,empcut,place,msaint,msaname,mdef):
    
    print("%s: %s" %(msaint, msaname))

    #Bring in block list
    blks = pd.io.parsers.read_csv('output/blocks/blocks_%s/blks_%s_%s.csv' %(mdef,mdef,msaint))

    #Convert to string
    blks['GEOID10'] = blks.GEOID10.apply(lambda x: "%015d" %x)
        # In New York City, CZ version has two islands not connected to any other blocks
        # In the middle of the Hudson, which is the border of the CZ.

    if mdef == 'cz_1990' and msaint == '19400':
        blks = blks[blks['GEOID10'] != '360610001001001']
        blks = blks[blks['GEOID10'] != '360610001001000']

    #Confirm no jobs over 500000
    assert blks.jdens.max() < 500001

    #Make named jobs variable
    blks['jobs'] = blks.C000

    #Get spatial weights
    try:
        wtsb = ps.open('output/weights/%s/b%s/b%s_%s.gal' %(mdef,bufthresh,bufthresh,msaint),'r').read()
    except IOError:
        #bring in unprojected shapefile and project now
        getbufweights(bufthresh,mdef,msaint,utms)
        wtsb = ps.open('output/weights/%s/b%s/b%s_%s.gal' %(mdef,bufthresh,bufthresh,msaint),'r').read()


    
    #Identify CBDs - the 1 at the end means the true value. For random reshuffling, run the commented loop below
    groupcbds(blks,wtsb,msaint,place,mdef,1)
    
    
def getbufweights(bufthresh,mdef,msaint,utms):
    gpd = gp.read_file('data/shapes_%s/shp_%s_%s/shp_%s_%s.shp' %(mdef,mdef,msaint,mdef,msaint))
    
    totbounds = gpd.geometry.total_bounds
    avglon = (totbounds[0] + totbounds[2]) / 2 
    zone = np.ceil((avglon + 180) / 6)
    utmepsg = utms[zone]
    assert len(utmepsg) == 5
    
    #Project into meters
    gpd.crs = {'init': 'epsg:4269'}
    gpd['geometry'] = gpd.geometry.to_crs(epsg = utmepsg)
    
    #Make buffer and non-buffer versions
    buf = gpd[['GEOID10','geometry']]
    buf.geometry = buf.buffer(bufthresh)
    #buf.to_file('data/shapes_%s/shp_%s_%s/shp_%s_%s_buf.shp' %(mdef,mdef,msaint,mdef,msaint)) 
    
    base = gpd[['GEOID10','geometry']]
    
    #spatial join them - getting everything within dist threshold meters
    joined = gp.sjoin(base,buf, how = 'left', op = 'intersects')
    joined = joined[joined.GEOID10_left != joined.GEOID10_right]
    
    joined[['GEOID10_left','GEOID10_right']].to_csv('output/weights/b%s/b%s_%s.csv' %(bufthresh,bufthresh,msaint))
    
    #Convert to weights
    jdict = {k: list(v) for k,v in joined.groupby("GEOID10_left")["GEOID10_right"]}
    wtsb = ps.weights.weights.W(jdict)  
    
    #Write out
    gal = ps.open('output/weights/%s/b%s/b%s_%s.gal' %(mdef,bufthresh,bufthresh,msaint),'w')
    gal.write(wtsb)
    gal.close()
    
    
def groupcbds(blks,wtsb,msaint,place,mdef,outblocks):
    #Sort in descending order so that first CBDs have highest job densities
    blks.sort_values(['jdens','GEOID10'],ascending = False, inplace = True)

    #Get a data frame of just the employment blocks. Note that they must have more than 0 jobs in addition to whatever ratio threshold is being used
    cbdb = blks[(blks.jobs > blks.pop10 * empcut) & (blks.jobs > 0)]

    #Make variable to aggregate to get block count
    cbdb['cntblks'] = 1

    cbdb.set_index('GEOID10', inplace = True)

    #Keep a list of candidate blocks that aren't yet classified
    unclassed = list(cbdb.index)
    classed = {}

    #Start CBD numbers
    cbdnum = 1

    while len(unclassed) > 0: #sum(cbdb.cbd == '') >0:

        #Get the earliest block that isn't yet classified
        startblk = unclassed[0] 
         
        #Start list of blocks in the new business district with just that block
        blklist = [startblk]
        
        #Add the block to the dictionary of classified blocks, 
        classed[startblk] = cbdnum
        unclassed.remove(startblk)
        
        #Get list of neighbors
        neighbors = list(set(wtsb.neighbors[startblk]))
                	    
        #Figure out which neighbors are also employment blocks
        candneighbs = list(set(unclassed).intersection(neighbors)) 
        
        #Loop through all neighbors, assign them to this BD, remove them from the list of unclassified employment blocks
        for i in candneighbs: 
            classed[i] = cbdnum
            blklist.append(i)
            unclassed.remove(i)

        #Now, add all the recently assigned neighbors to the list of candidate blocks to iterate through and assign to this BD     
        candblks = []
        candblks = candblks + candneighbs

        #Go through each of the neighbors and add their neighbors to the CBD
        while len(candblks) >0 :

            #Go through each candidate block and take it off the list
            nextblk = candblks.pop()
            
            #Get neighbors of this candidate block
            neighbors = list(set(wtsb.neighbors[nextblk]))
        
            #Get the neighbors that are also employment blocks
            candneighbs = list(set(unclassed).intersection(neighbors)) #I think it's ok to use the full list because everything should be symmetric 
            
            #Add to ongoing list
            candblks = candblks + candneighbs

            #New neighbors get classified in CBD

            for i in candneighbs: 
                classed[i] = cbdnum
                blklist.append(i)
                unclassed.remove(i)

        #Finally, move to next cbd
        cbdnum +=1


    #Add cbd number to data frame of employment blocks and export block to BD lookup
    cbdb['cbd'] = pd.DataFrame.from_dict(classed, orient = 'index')
    cbdb[['cbd','pop10','jobs']].to_csv('output/bizdist_%s/%s/bd_blks/bd_blks_%s.csv' %(place,mdef,msaint), index = True)

    #print('5.5:%s' %str(datetime.now()))    
    #Aggregate employment blocks by CBD number
    cbds = cbdb[['cbd','jobs','ALAND10','cntblks','pop10']].groupby('cbd').aggregate(np.sum)
    cbds.sort_values(by = ['jobs','cntblks','ALAND10','pop10'], ascending = False, inplace = True)

    if outblocks == 1:
        cbds.to_csv('output/bizdist_%s/%s/bd_stats/bd_stats_%s.csv' %(place,mdef,msaint))

    if outblocks > 1:
        cbds.to_csv('output/bizdist_%s/%s/bd_stats/randomize/bd_stats_random_%s_%s.csv' %(place,mdef,msaint,outblocks))


    #Export shapefile of cbds
    shp = gp.read_file('data/shapes_%s/shp_%s_%s/shp_%s_%s.shp' %(mdef,mdef,msaint,mdef,msaint))

    cbblks = shp[shp.GEOID10.isin(cbdb.index)]
    cbblk = cbblks.merge(cbdb, left_on = 'GEOID10', right_index = True)

    #Dissolve
    consol = cbblk.dissolve(by = 'cbd')
    consol['cbd'] = consol.index

    consol = consol.merge(cbds, left_index = True, right_index = True, suffixes = ['_drop',''])
    consol = gp.GeoDataFrame(consol[['cbd','lat','lon','jobs','cntblks','ALAND10','geometry']])
    consol.crs = {'init': 'epsg:4269'}

    consol.to_file('output/bizdist_%s/%s/bd_shapes/bds_%s_%s_%s' %(place,mdef,msaint,place,mdef)) #,mdef,msaint))

    
    
    
# Now loop through msa definitions and run everything
msadefs = ['cz_1990','msa_2018','cz_1990'] 

    
for mdef in msadefs:
    
    #Weights directory
    try:
        os.stat("output/weights/%s" %mdef)
    except:
        os.mkdir("output/weights/%s" %mdef)

    try:
        os.stat("output/weights/%s/b%s" %(mdef,bufthresh))
    except:
        os.mkdir("output/weights/%s/b%s" %(mdef,bufthresh))
        
    # Bring in County to MSA lookup
    msalist = pd.io.parsers.read_csv('data/reference/%s.csv' %mdef, encoding ='ISO-8859-1') #, 'utf-8
    
    if mdef in ['msa_2013','msa_2018']:
        
        if mdef == 'msa_2018':
            startname = msalist.columns[0]
            msalist.rename(columns = {startname:'CBSA Code'},inplace = True)
        
        msalist['msa'] = msalist['CBSA Code'].apply(str)
        msalist['msaname'] = msalist['CBSA Title']
        msalist['fips_st'] = msalist['FIPS State Code'].apply(lambda x: "%02.f" %(x))
        msalist['fips_cnty'] = msalist['FIPS County Code'].apply(lambda x: "%03.f" %(x))
        msalist['stcnty'] = msalist['fips_st'] + msalist['fips_cnty']
    
    elif mdef in ['cz_1990']:
        msalist['stcnty'] = msalist['County FIPS Code'].apply(lambda x: "%05.f" %x)
        msalist['fips_st'] = msalist['stcnty'].apply(lambda x: x[:2])
        msalist['fips_cnty'] = msalist['stcnty'].apply(lambda x: x[2:5])
        msalist['msa'] = msalist['CZ90'].apply(str)
        msalist['msaname'] = msalist['Name of largest place in Commuting Zone'].apply(str)
    
    #Drop Puerto Rico
    msalist = msalist[msalist['fips_st'] != '72']
    
     
    #MSAs for looping
    msadf = msalist.sort_values(by = ['msa'])
    msadf = msadf.drop_duplicates('msa')
    msadf.reset_index(inplace =True)
    mlist = msadf['msa']   
    
    loadmsas(bufthresh,empcut,mlist,msalist,mdef)
        

