#Assign business districts into categories based on size and density

library(Hmisc)
library(scales)
library(stringr)
library(tidyverse)

homedir = '<set to directory>'
setwd(homedir)

#Set thresholds for number of blocks to be a large district and density to be an urban core
blockcut = 5
denscut = 10000
jobscut = 500

#Loop through MSA definitions
msadefs = c('msa_2013','msa_2018','cz_1990')

for(mdef in msadefs){
  mdef_clean = str_split(mdef,'_',simplify = T)[,1] %>% str_to_upper()
  
  #Get MSA stats
  msas = read.csv(paste('output/msapop/pop_',mdef,'.csv',sep = ''))
  msas$shortname = str_split(msas$msaname,'[-,/]',simplify = T)[,1]
  msas$st = str_split(msas$msaname,'[,]',simplify = T)[,2]
  msas$st = str_replace(msas$st, ' ','')
  msas$st = str_split(msas$st,'-',simplify = T)[,1]
  msas$shortname_st = paste(msas$shortname,msas$st, sep = ', ')
  
  #Get 100 largest MSAs
  m100 = msas %>% arrange(-pop)
  m100 = m100[1:100,'msa']
  msas$top100 = msas$msa %in% c(m100)
  msas$all = T
  
  #If there are multiple specifications, can loop through them here
  #Specification
  for(place in c('bizdist_b15_r2.0')){ #,'bizdist_b15_d0.5','bizdist_b15_r2.0_scan3'
  
    dir.create(paste('output/',place,'/',mdef,'/bd_cats',sep = ''))
    
    empcut = str_extract(place,'r[0-9]\\.[0-9]') %>% str_remove('r') %>% as.numeric()
    
    #Preserve parallel format with raw density cut
    if(is.na(empcut)) empcut = 1
    rescut = 1/empcut
  
    allcbds = NULL
    cbdlist = NULL
    
    #Loop through all the MSAs
    for(cb in msas$msa){
      if(cb == '34114') next()
      
      msaname = msas[msas$msa == cb,'msaname']
      sname = msas[msas$msa == cb,'shortname']
      print(paste(cb,msaname,sname))
      
      #Bring in block to cbd link
      cbdlink = read.csv(paste('output/',place,'/',mdef,'/bd_blks/bd_blks_',cb,'.csv',sep = ''),header = T)
      cbdlink = select(cbdlink,GEOID10,cbd) # names(cbdlink) = c('GEOID10','cbd','pop','')
      cbdlink$GEOID10 = str_pad(cbdlink$GEOID10,width = 15,pad = '0')
      
      #Bring in all blocks
      blks = read.csv(paste('output/blocks/blocks_',mdef,'/blks_',mdef,'_',cb,'.csv',sep = ''))
      blks$GEOID10 = str_pad(blks$GEOID10,width = 15,pad = '0')
      rowcheck = nrow(blks)
      blks = merge(blks, cbdlink, by = 'GEOID10', all = T)
      stopifnot(nrow(blks) == rowcheck) #Make sure merge is working
      
      #Make clean variable names
      #Convert area into numeric
      blks$ALAND10 = as.numeric(blks$ALAND10)
      blks$popvar = blks$pop10
      blks$jobtot = blks$C000
      
      #Confirm no blocks with zero land and positive jobs
      stopifnot((blks$ALAND10 == 0 & blks$C000 > 0) == F)
      
      #Confirm that density is topcoded
      stopifnot(blks$jdens <= 500000)
      
      #Drop unbuilt 
      blks = filter(blks, popvar > 0 | C000 > 0)
  
      #Classify into res, mixed, and employment
      if(place %in% c('bizdist_b15_r2.0','bizdist_b15_r2.0_scan3')){
      blks$ratio = blks$jobtot / blks$popvar
      blks[blks$popvar == 0 & blks$jobtot > 0, 'ratio'] = 100000
      blks$cat = 'None'
      blks[blks$ratio < rescut & blks$popvar > 0, 'cat'] = 'res'
      blks[blks$ratio > empcut & blks$jobtot > 0, 'cat'] = 'emp'
      blks[blks$ratio >= rescut & blks$ratio <= empcut & blks$popvar > 0 & blks$jobtot > 0, 'cat'] = 'mixed'
      }
      
      if(place == 'bizdist_b15_d0.5'){
        blks$cat = 'None'
        blks[is.na(blks$cbd),'cat'] = 'res'
        blks[!is.na(blks$cbd),'cat'] = 'emp'
      }
      
      #Collapse into CBDs plus one category each for mixed and res
      cbds = blks %>% group_by(cbd,cat) %>% summarise(
        area = sum(ALAND10) / 1000000, pop = sum(popvar), lat = wtd.mean(lat, jobtot),
        lon = wtd.mean(lon, jobtot), jobs = sum(jobtot), 
        mdens = wtd.quantile(jdens, jobtot, .5),p25dens = wtd.quantile(jdens, jobtot, .25), cnt = length(GEOID10)) %>% data.frame()
      
      #Assign each CBD to its category 
      cbds$catdet = cbds$cat
      cbds$tdens = cbds$jobs / cbds$area #* 1000000
      cbds[cbds$cnt < blockcut & cbds$jobs < jobscut, 'catdet'] = 'scatter'
      cbds[cbds$cnt < blockcut & cbds$jobs >= jobscut, 'catdet'] = 'ind'
      cbds[cbds$cnt >= blockcut & cbds$p25dens >= denscut, 'catdet'] = 'core'
      cbds[cbds$cnt >= blockcut & cbds$p25dens < denscut, 'catdet'] = 'suburb'
      
      #Assign overall categories
      cbds[cbds$cat == 'mixed', 'catdet'] = 'mixed'
      cbds[cbds$cat == 'res', 'catdet'] = 'res'
      
      #Sort by jobs, then block count, then area, then population 
      cbds = cbds %>% arrange(-jobs,-cnt,-area,-pop)
      
      #Output CBD categories and densities
      write.csv(cbds %>% arrange(-jobs),paste('output/',place,'/',mdef,'/bd_cats/bd_cats_',mdef,'_',cb,'.csv',sep = ''), row.names = F)
      
      #Get total statistics by category for each MSA
      cbcats = cbds %>% group_by(catdet) %>% summarise(area = sum(area), jobs = sum(jobs), pop = sum(pop), num = length(cbd), blks = sum(cnt)) %>% data.frame()
      totmsa = blks %>% summarise(area = sum(ALAND10), jobs = sum(jobtot), pop = sum(popvar)) %>% data.frame()
      cbcats$msa = cb
      totmsa$msa = cb
      
      #Add to national list
      cbcats = merge(cbcats, totmsa, by = 'msa', suffixes = c('','_tot'))
      allcbds = rbind(allcbds, cbcats)
      
      #Get national list of all buisness districts 
      cbdexp = cbds %>% filter(cat == 'emp') %>% select(area,pop,jobs,cnt,catdet) %>% arrange(-jobs)
      cbdexp$msarank = 1:nrow(cbdexp)
      cbdexp$msa = cb
      cbdexp$sname = sname
      cbdlist = rbind(cbdlist, cbdexp)
    }
    
    #Get overall national statistics
    national = allcbds %>% group_by(catdet) %>% summarise(area = sum(area), jobs = sum(jobs), pop = sum(pop),blks = sum(blks)) %>% data.frame() #summarise(area = max(area_tot), jobs = max(jobs_tot), pop = max(pop_tot)) %>% data.frame()
    nattots = national %>% summarise(jobs = sum(jobs), pop = sum(pop), area = sum(area), blks = sum(blks)) %>% data.frame()
    nattots$catdet = 'tot'
    national = rbind(national,nattots)
    national$area_pct = national$area / nattots$area
    national$jobs_pct = national$jobs / nattots$jobs
    national$pop_pct = national$pop / nattots$pop
    national$blks_pct = national$blks / nattots$blks
    national$catclean = factor(national$catdet, levels = c('res','mixed','core','suburb','ind','scatter','tot'),labels = c('Residential','Mixed','Urban core','Suburban','Independent','Scattered','Total'))
    national = national %>% select(catclean,starts_with('area'),starts_with('jobs'),starts_with('pop'),starts_with('blks')) %>% arrange(catclean)
    
    write.csv(national,paste('output/',place,'/',mdef,'/table_s3_bdcats_tot_',mdef,'_all.csv',sep = ''),row.names = F)
    
    #Get overall statistics for top 100
    t100 = allcbds %>% filter(msa %in% m100) %>% group_by(catdet) %>% summarise(area = sum(area), jobs = sum(jobs), pop = sum(pop),blks = sum(blks)) %>% data.frame() #summarise(area = max(area_tot), jobs = max(jobs_tot), pop = max(pop_tot)) %>% data.frame()
    t100tots = t100 %>% summarise(jobs = sum(jobs), pop = sum(pop), area = sum(area), blks = sum(blks)) %>% data.frame()
    t100tots$catdet = 'tot'
    t100 = rbind(t100,t100tots)
    t100$area_pct = t100$area / t100tots$area
    t100$jobs_pct = t100$jobs / t100tots$jobs
    t100$pop_pct = t100$pop / t100tots$pop
    t100$blks_pct = t100$blks / t100tots$blks
    t100$catclean = factor(t100$catdet, levels = c('res','mixed','core','suburb','ind','scatter','tot'),labels = c('Residential','Mixed','Urban core','Suburban','Independent','Scattered','Total'))
    t100 = t100 %>% select(catclean,starts_with('area'),starts_with('jobs'),starts_with('pop'),starts_with('blks')) %>% arrange(catclean)
    
    write.csv(t100,paste('output/',place,'/',mdef,'/table_s3_bdcats_tot_',mdef,'_top100.csv',sep = ''),row.names = F)
    
    
    #Output list of 500 largest bds in the country 
    natrank = cbdlist %>% arrange(-jobs)
    natrank$nrank = 1:nrow(natrank)
    write.csv(natrank[1:500,] ,paste('output/',place,'/',mdef,'/table_s2_nat_rank.csv',sep = ''),row.names = F)
    
    ## Graphs
    #Make graph of breakdown of employment across type of BD by MSA
    allcbds$jobs_pct = allcbds$jobs / allcbds$jobs_tot
    allcbds$area_pct = allcbds$area / allcbds$area_tot
    allcbds$pop_pct = allcbds$pop / allcbds$pop_tot
    
    write.csv(allcbds,paste('output/',place,'/',mdef,'/bdcats_national_',mdef,'.csv',sep = ''),row.names = F)
  
    #Make graphs 
    pltcbds = merge(allcbds, msas[,c('msa','shortname','shortname_st','all','top100')], by = 'msa')
    
    for(samp in c('all','top100')){
      mdat = pltcbds[pltcbds[,samp] == T,]
      mdat$catdet = factor(mdat$catdet, levels = c('res','mixed','scatter','ind','suburb','core'),labels = c('Residential','Mixed','Scattered','Independent','Suburban','Urban Core'))
      mdat$outname = factor(mdat$shortname_st,levels = rev(unique(mdat$shortname_st)))
      
      if(samp == 'all'){
        pdf(paste('output/',place,'/',mdef,'/bdcats_nat_',mdef,'_',samp,'.pdf',sep = ''),height = 25)
      }
      
      if(samp == 'top100'){
        pdf(paste('output/',place,'/',mdef,'/bdcats_nat_',mdef,'_',samp,'.pdf',sep = ''),height = 13,width = 8)
      }
  
      p = ggplot(mdat, aes(y = jobs_pct, x = outname, fill = catdet)) +
        geom_bar(stat = 'identity') +
        scale_fill_grey(start = .90, end = .30, name = '')+
        coord_flip() +
        scale_x_discrete(name = '') +
        theme_bw() +
        ggtitle(paste('Breakdown of employment across type of block by',mdef_clean)) +
        scale_y_continuous(label = percent, name = paste('Percent of',mdef_clean,'employment'))
      
      print(p)
      dev.off()
    }
  }
}
    