#Make satellite map of NYC

library(maptools)
library(rgeos)
library(tidyverse)
library(rgdal)
library(ggthemes)
library(ggmap)
library(stringr)
library(Hmisc)
library(scales)

homedir = '<set to directory>'
setwd(homedir)

#Set up Google
apikey = '<sign up for google key>'
register_google(key = apikey)

# Loop through MSA definitions
msadefs = c('msa_2013','msa_2018','cz_1990')

for(mdef in msadefs){
  mdef_clean = str_split(mdef,'_',simplify = T)[,1] %>% str_to_upper()
  
  #Get MSA stats
  msas = read.csv(paste('output/msapop/pop_',mdef,'.csv',sep = ''))
  msas$shortname = str_split(msas$msaname,'[-,/]',simplify = T)[,1]
  msas$st = str_split(msas$msaname,'[,]',simplify = T)[,2]
  msas$st = str_replace(msas$st, ' ','')
  msas$st = str_split(msas$st,'-',simplify = T)[,1]
  msas$shortname_st = paste(msas$shortname,msas$st, sep = ', ')
  
  #Loop through MSAs in order of population just in case we run out of API calls
  msas = msas %>% arrange(-pop)

  #Loop through MSAs
  for(i in 115:nrow(msas)){ #1
    
    msa = msas[i,'msa']
    sname = msas[i,'shortname_st']
    longname = msas[i,'msaname']
    print(paste(msa, sname,i))
    
    #Get basemap first to limit calls to google
    basemap= get_map(location = sname, maptype = 'satellite', source = 'google', zoom = 10)
    basemapz= get_map(location = sname, maptype = 'satellite', source = 'google', zoom = 11)
    
    #Make background transparent
    bosbase_att = attributes(basemap)
    basemapt <- matrix(adjustcolor(basemap, alpha.f = 0.5), nrow = nrow(basemap))
    
    attributes(basemapt) = bosbase_att
    
    bosbase_att = attributes(basemapz)
    basemapzt <- matrix(adjustcolor(basemapz, alpha.f = 0.5), nrow = nrow(basemapz))
    
    attributes(basemapzt) = bosbase_att
    
    #Loop through specifications if present 
    for(place in c('b15_r2.0')){ #,'b15_d0.5','b15_r2.0_scan3'
        
      dir.create(paste('output/bizdist_',place,'/',mdef,'/bd_maps/satmaps',sep = ''))
      dir.create(paste('output/bizdist_',place,'/',mdef,'/bd_maps/maps_',mdef,'_',msa,'_',sname,sep = ''))
      
      #Get CBDs
  
      #Old shape name convention in main specification
      placeshape = place
      if(place == 'b15_r2.0') placeshape = 'b15_r2.0_j0'
      
      #Naming output
      shplist = list.files(paste('output/bizdist_',place,'/',mdef,'/bd_shapes/bds_',msa,'_',placeshape,'_',mdef,'/',sep = ''))
      shpname = grep(pattern = 'shp',x = shplist,value = T)
      cbds = readOGR(paste('output/bizdist_',place,'/',mdef,'/bd_shapes/bds_',msa,'_',placeshape,'_',mdef,'/',shpname,sep = ''))
      cbds@data$id = row.names(cbds@data)
      cbds.points <- fortify(cbds, region = "id")
      
      #Bring in CBD categories
      ccats = read.csv(paste('output/bizdist_',place,'/',mdef,'/bd_cats/bd_cats_',mdef,'_',msa,'.csv',sep = ''))
      ccats = filter(ccats, cat == 'emp') #remove miscategorized mixed districts
      
      #Merge on cbd numbers to spatial points
      cbdptskey = merge(cbds.points, cbds@data[,c('id','cbd','ALAND10')], by = 'id', all.x = T)
      cbdcat = merge(cbdptskey, ccats[,c('cbd','catdet')], by = 'cbd', all.x = T) %>% arrange(order)
      
      #There are some CBDs that don't merge beause they have zero land area adn we dropped those. Filter them here
      stopifnot(!is.na(cbdcat$catdet) ) #| cbdcat$ALAND10 == 0

      # Make clean names
      cbdcat$cleancat = factor(cbdcat$catdet,levels = c('core','ind','suburb','scatter'),labels = c('Urban Core','Independent','Suburban','Scattered'))
      
      #Create maps
      
      #Zoomed in
      jpeg(paste('output/bizdist_',place,'/',mdef,'/bd_maps/maps_',mdef,'_',msa,'_',sname,'/bd_sat_',msa,'_',sname,'_central.jpeg',sep = ''),height = 9, width = 8.5,units = 'in',res = 400) #width = 3000, height = 2300 )
      p = ggmap(basemapzt,color = 'bw') +
        geom_path(data = filter(cbdcat, hole == F), aes(long, lat, group = group),size =.2) +
        geom_polygon(data = filter(cbdcat, hole == F), aes(long, lat, group = group,fill = cleancat),alpha = .7) +
        scale_fill_grey(breaks = c('Urban Core','Suburban','Independent','Scattered'),start = 0,end = 1)+
        theme_map()+
        guides(fill = guide_legend(title = 'Business District Type',nrow = 1, byrow = T,
          override.aes = list(color = "black",size = .2),legend.position = 'bottom')) +
        ggtitle(paste('Business districts of the',longname,mdef_clean)) +
        theme(legend.position = 'bottom',legend.justification = 'center') 
      
      print(p)
      dev.off()
      
      #Zoomed out
      jpeg(paste('output/bizdist_',place,'/',mdef,'/bd_maps/maps_',mdef,'_',msa,'_',sname,'/bd_sat_',msa,'_',sname,'.jpeg',sep = ''),height = 9, width = 8.5,units = 'in',res = 400) #width = 3000, height = 2300 )
      p = ggmap(basemapt,color = 'bw') +
        geom_path(data = filter(cbdcat, hole == F), aes(long, lat, group = group),size =.2) +
        geom_polygon(data = filter(cbdcat, hole == F), aes(long, lat, group = group,fill = cleancat),alpha = .7) +
        scale_fill_grey(breaks = c('Urban Core','Suburban','Independent','Scattered'),start = 0,end = 1)+
        theme_map()+
        guides(fill = guide_legend(title = 'Business District Type',nrow = 1, byrow = T,
                                   override.aes = list(color = "black",size = .2),legend.position = 'bottom')) +
        ggtitle(paste('Business districts of the',longname,mdef_clean)) +
        theme(legend.position = 'bottom',legend.justification = 'center')
      
      print(p)
      dev.off()
        
    
      # Make special new york city map for Figure 4 of main text
      if(mdef == 'msa_2013' & msa == 35620){
        nycent =c(-73.994415,40.754713) #984439
        
        #Get basemap first to limit calls to google
        basemapz= get_map(location = nycent, maptype = 'satellite', source = 'google', zoom = 11) #-73.984010,40.755038
        
        #Make background transparent
        bosbase_att = attributes(basemapz)
        basemapzt <- matrix(adjustcolor(basemapz,alpha.f = 0.5), nrow = nrow(basemapz))
        
        attributes(basemapzt) = bosbase_att
        
        jpeg(paste('output/bizdist_',place,'/',mdef,'/bd_maps/fig4_bd_sat_',msa,'_',place,'_central.jpeg',sep = ''),height = 6.8, width = 9,units = 'in',res = 400) #width = 3000, height = 2300 )
        p = ggmap(basemapzt,color = 'bw') +
          geom_path(data = filter(cbdcat, hole == F), aes(long, lat, group = group),size =.2)+
          geom_polygon(data = filter(cbdcat, hole == F), aes(long, lat, group = group,fill = cleancat),alpha = .7) +
          scale_fill_grey(breaks = c('Urban Core','Suburban','Independent','Scattered'),start = 0,end = 1)+
          scale_y_continuous(limits = c(40.63,40.86),expand = c(0,0)) + # scale_alpha_discrete(breaks = c('Independent','Scattered','Urban Core','Suburban'),pallete = c(.5,0,.5,.5)) +
          theme_map()+
          guides(fill = guide_legend(title = 'Business District Type',nrow = 1, byrow = T,
                                     override.aes = list(color = "black",size = .2),legend.position = 'bottom')) +
          theme(legend.position = 'bottom',legend.justification = 'center') 
        print(p)
        dev.off()
        
        
      }
      
      
    } #place
  } #MSA
} # mdef

    