# Program to identify CBDs and compute concentration index for each. 
# Outputs Figure 2 from main text and Figure S2 in supplementary materials

library(Hmisc)
library(scales)
library(stringr)
library(fields) #For rdist.earth function
library(tidyverse)

homedir = '<set to directory>'
setwd(homedir)

# Loop through MSA definitions
msadefs = c('msa_2013','msa_2018','cz_1990')

for(mdef in msadefs){
  mdef_clean = str_split(mdef,'_',simplify = T)[,1] %>% str_to_upper()
  
  #Get MSA stats
  msas = read.csv(paste('output/msapop/pop_',mdef,'.csv',sep = ''))
  msas$shortname = str_split(msas$msaname,'[-,/]',simplify = T)[,1]
  msas$st = str_split(msas$msaname,'[,]',simplify = T)[,2]
  msas$st = str_replace(msas$st, ' ','')
  msas$st = str_split(msas$st,'-',simplify = T)[,1]
  msas$shortname_st = paste(msas$shortname,msas$st, sep = ', ')
  
  #Identify the 100 largest MSAs
  m100 = msas %>% arrange(-pop)
  m100 = m100[1:100,'msa']
  
  msas = arrange(msas,-pop)
  
  #If there are multiple specifications, can loop through them here
  for(place in c('bizdist_b15_r2.0')){ #bizdist_b15_r2.0_scan3
    
    #Make new output directories
    dir.create(paste('output/',place,'/',mdef,'/metro_centers',sep = ''))
    dir.create(paste('output/',place,'/',mdef,'/metro_centers/concentration_graphs',sep = ''))

    #Loop through MSAs
    metrocents = NULL
    
    for(i in 1:nrow(msas)){
      msa = msas[i,'msa']
      if(msa == '34114') next()
      cname = msas[i,'shortname_st']
      sname = msas[i,'shortname']
      mname = msas[i,'msaname']
      print(paste(msa, sname))
      
      #Get categorized CBD list
      cbds = read.csv(paste('output/',place,'/',mdef,'/bd_cats/bd_cats_',mdef,'_',msa,'.csv',sep = ''))
      
      #Get total employment and rank of center
      totemp = sum(cbds$jobs)
      cbds$jobspct = cbds$jobs / totemp
      cbds$rank = 1:nrow(cbds)
      
      #Determine whether there are urban cores with >1% of total jobs
      cores = filter(cbds, catdet == 'core', jobspct > .01)
      
      #If there are cores, take the largest
      if(nrow(cores) > 0) metrocent = cores[1,]
      
      #If there are not, take largest center
      if(nrow(cores) == 0) metrocent = filter(cbds, cat == 'emp')[1,]
      
      #Now do concentration index
      #Bring in all blocks
      blks = read.csv(paste('output/blocks/blocks_',mdef,'/blks_',mdef,'_',msa,'.csv',sep = ''))
      
      #Drop unbuilt 
      blks = filter(blks, pop10 > 0 | C000 > 0)
      
      #Confirm all built blocks have positive land area
      stopifnot(blks$ALAND10 > 0)
      
      #Switch ALAND to numeric
      blks$ALAND10 = as.numeric(blks$ALAND10)
      
      #Distance in miles
      blks$dist = rdist.earth(x1 = blks[,c('lon','lat')],x2 = metrocent[,c('lon','lat')],miles = T) %>% as.vector()
      
      #Sort by distance
      blks = arrange(blks,dist)
      blks$cumpop = cumsum(blks$pop10) / sum(blks$pop10)
      blks$cumjob = cumsum(blks$C000) / sum(blks$C000)
      blks$distlag = c(blks$dist[1],diff(blks$dist))
    
      #Overall area under curve - I think I'm doing this right: weight by each distance out and divide by total distance * 1
      metrocent$con_j_d = sum(blks$cumjob * blks$distlag) / sum(blks$distlag)
      metrocent$con_p_d = sum(blks$cumpop * blks$distlag) / sum(blks$distlag)
      
      #Plot distance concentration curve for each MSA
      pdta = blks %>% select(dist,cumpop,cumjob) %>% gather(key = 'key',value = 'value',-dist)
      pdta$key = factor(pdta$key, levels = c('cumjob','cumpop'), labels = c('Employment','Population'))
      pdf(paste('output/',place,'/',mdef,'/metro_centers/concentration_graphs/con_',msa,'_',sname,'_dist.pdf',sep = ''),width = 5.5)
      gplt = ggplot(pdta) +
        geom_line(aes(x = dist,y = value,col = key)) +
        theme_bw()+
        scale_color_grey(name = '') +
        scale_x_continuous(name = 'Distance from CBD (miles)') +
        scale_y_continuous(labels = percent,name = 'Cumulative percentage of jobs/residents') +
        theme(legend.position = 'bottom') +
        ggtitle(paste('Cumulative distribution of employment and population by\ndistance from CBD, ',cname,' ',mdef_clean,sep = ''))
      print(gplt)
      dev.off()
      
      if(msa == '14460' & mdef == 'msa_2013') {
        gplt = gplt + 
          ggtitle('Distance from CBD') +
          theme(plot.title = element_text(hjust = 0.5))
        pdf(paste('output/',place,'/',mdef,'/metro_centers/fig2_',msa,'_',sname,'_dist.pdf',sep = ''),width = 5.5)
        print(gplt)
        dev.off()
        
        }
      
      #Now do area version
      blks$pdens = blks$pop10 / blks$ALAND10
      blks$jdens = blks$C000 / blks$ALAND10
      
      #Get cumulative density of jobs
      cumjobs = blks %>% select(jdens,ALAND10, C000) %>% arrange(-jdens)
      cumjobs$landpct = cumjobs$ALAND10 / sum(cumjobs$ALAND10) #Should sum to 1 but make sure
      cumjobs$lpct = cumsum(cumjobs$ALAND10) / sum(cumjobs$ALAND10)
      cumjobs$pct = cumsum(cumjobs$C000) / sum(cumjobs$C000)
      cumjobs$key = 'Employment'
      # cumjobs = filter(cumjobs, pct < .99)
      
      cumpop = blks %>% select(pdens,ALAND10, pop10) %>% arrange(-pdens)
      cumpop$landpct = cumpop$ALAND10 / sum(cumpop$ALAND10)
      cumpop$lpct = cumsum(cumpop$ALAND10) / sum(cumpop$ALAND10)
      cumpop$pct = cumsum(cumpop$pop10) / sum(cumpop$pop10)
      cumpop$key = 'Population'

      #Get overall area under curve - sum weighted by land percent / total land
      metrocent$con_j_a = sum(cumjobs$pct * cumjobs$landpct) / sum(cumjobs$landpct)
      metrocent$con_p_a = sum(cumpop$pct * cumpop$landpct) / sum(cumpop$landpct)
      
      pdta = rbind(cumjobs[,c('key','lpct','pct')],cumpop[,c('key','lpct','pct')]) %>% data.frame()
      
      pdf(paste('output/',place,'/',mdef,'/metro_centers/concentration_graphs/con_',msa,'_',sname,'_dens.pdf',sep = ''),height = 7,width = 5.5)
      gplt = ggplot(pdta) +
        geom_line(aes(x = lpct,y = pct,col = key)) +
        theme_bw()+
        scale_color_grey(name = '') +
        scale_x_continuous(labels = percent,name = 'Cumulative percentage of built land area') +
        scale_y_continuous(labels = percent,name = 'Cumulative percentage of jobs/residents') +
        theme(legend.position = 'bottom') +
        ggtitle(paste('Cumulative distribution of employment and population by\ncumulative land area, ',cname,' ',mdef_clean,sep = ''))
      print(gplt)
      dev.off()
      
      if(msa == '14460' & mdef == 'msa_2013') {
        gplt = gplt + 
          ggtitle('Density') +
          theme(plot.title = element_text(hjust = 0.5))
        pdf(paste('output/',place,'/',mdef,'/metro_centers/fig2_',msa,'_',sname,'_dens.pdf',sep = ''),width = 5.5)
        print(gplt)
        dev.off()
        
        #What percent of jobs and people on densest 10% of land - mention on page 11
        filter(cumjobs, lpct <= .1) %>% summarise(jobs = sum(C000),landpct = sum(landpct))
        #2098367 jobs on 0.0999997
        sum(cumjobs$C000) #2270359 total jobs
        2098367 / 2270359 #0.9242446 of jobs
        
        filter(cumpop, lpct <= .1) %>% summarise(pop = sum(pop10),landpct = sum(landpct))
        #2986132 people on 0.0999976 of land
        sum(cumpop$pop10) #4552402 total people
        2986132 / 4552402 #0.6559465
        
        
      }
      
      #Add to output
      metrocent$msa = msa
      metrocent$cname = cname
      metrocent$sname = sname
      metrocents = rbind(metrocents, metrocent)
      
    }
    
    #Output data
    metrocents$totemp = metrocents$jobs / metrocents$jobspct
    write.csv(metrocents,paste('output/',place,'/',mdef,'/metro_centers/metcents_',mdef,'_',place,'.csv',sep = ''),row.names = F)
    
    #Make graphs with all cities 
    
    #First do all 917
    #Distance
    pdf(paste('output/',place,'/',mdef,'/metro_centers/con_dist_all.pdf',sep = ''),height = 5)
    gplt = ggplot(metrocents,aes(x = con_p_d, y = con_j_d, label = sname)) +
                    geom_text(size = 2) +
      scale_x_continuous(limits = c(0.5,1),labels = percent, name = 'Population') +
      scale_y_continuous(limits = c(0.5,1), labels = percent, name = 'Employment') +
      geom_abline(slope = 1, intercept = 0, lty = 2, alpha = .5) +
      theme_bw() +
      ggtitle(paste('Distance concentration of population and employment, all ',mdef_clean,'s',sep = ''))
    print(gplt)
    dev.off()
    
    #Density
    pdf(paste('output/',place,'/',mdef,'/metro_centers/con_dens_all.pdf',sep = ''),height = 5)
    gplt = ggplot(metrocents,aes(x = con_p_a, y = con_j_a, label = sname)) +
      geom_text(size = 2) +
      scale_x_continuous(limits = c(0.8,1),labels = percent, name = 'Population') +
      scale_y_continuous(limits = c(0.8,1), labels = percent, name = 'Employment') +
      geom_abline(slope = 1, intercept = 0, lty = 2, alpha = .5) +
      theme_bw() +
      ggtitle(paste('Density concentration of population and employment, all ',mdef_clean,'s',sep = ''))
    print(gplt)
    dev.off()
    
    #Now do top 100
    #Distance
    top100 = filter(metrocents, msa %in% m100) 
    
    pdf(paste('output/',place,'/',mdef,'/metro_centers/con_dist_top100.pdf',sep = ''),height = 5)
    gplt = ggplot(top100,aes(x = con_p_d, y = con_j_d, label = sname)) +
      geom_text(size = 2) +
      scale_x_continuous(limits = c(0.5,1),labels = percent, name = 'Population') +
      scale_y_continuous(limits = c(0.5,1), labels = percent, name = 'Employment') +
      geom_abline(slope = 1, intercept = 0, lty = 2, alpha = .5) +
      theme_bw() +
      ggtitle(paste('Distance concentration of population and employment, 100 largest ',mdef_clean,'s',sep = ''))
    print(gplt)
    dev.off()
    
    pdf(paste('output/',place,'/',mdef,'/metro_centers/con_dens_top100.pdf',sep = ''),height = 5)
    gplt = ggplot(top100,aes(x = con_p_a, y = con_j_a, label = sname)) +
      geom_text(size = 2) +
      scale_x_continuous(limits = c(0.8,1),labels = percent, name = 'Population') +
      scale_y_continuous(limits = c(0.8,1), labels = percent, name = 'Employment') +
      geom_abline(slope = 1, intercept = 0, lty = 2, alpha = .5) +
      theme_bw() +
      ggtitle(paste('Density concentration of population and employment, 100 largest ',mdef_clean,'s',sep = ''))
    print(gplt)
    dev.off()

  } #place
} #mdef